C
C     Program SAAO322.FOR   release 2
C
C     Transforming SAAO Lucy UBV phot. output into Data B of HEC22
C     Correction for grey filters included
C
C     Version 2: October 11, 2005
C
C     Files needed:
C
C           1. night of measurements: output file of Lucy program
C              (all remarks are safely ignored)
C
C           2. tabsaao.con  ... contains names of stars in Lucy
C              program and corresponding Nos. in Table of objects of HEC22
C
C           3. saao.cal .... contains transparencies of the three
C              attenuating filters for the V, B and U magnitudes
C              in the order from the lighter to the heavier attenuation,
C              i.e. V(ND2), B(ND2), U(ND2), V(ND3), B(ND3), U(ND3),
C                   V(ND4), B(ND4), U(ND4) (free format)
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION TEX(10),TX(5),F(9)
      INTEGER*4 HV(3),S(999,3)
      INTEGER*2 NS(999),FIL(999),BAR(3)                                 Rel.2
      CHARACTER*1 ZN,ZK(3)
      CHARACTER*6 NGT
      CHARACTER*8 OBS
      CHARACTER*12 INP,OUT
C
      COMMON /DTC/ DT
C
C     Definition of special data values
C
      DATA ZK/1H*,1HH,1H/
      DATA BAR/3,2,1/
      DT=2D-8
C
C     Input formats
C
  101 FORMAT(A1)
  102 FORMAT(3A4,2I4)
  103 FORMAT(A1,3A4)
  104 FORMAT(A1,3I2,I3,I2,I3,I2,F9.0,I2)
C
C     Output formats
C
  201 FORMAT(A1,A12)
  202 FORMAT(5A8)
  203 FORMAT(F3.0,I3,I5,4H 311)
  204 FORMAT(I1,I4,2I3,F9.4,1X,F10.1)
  205 FORMAT(5H    0)
  206 FORMAT(10A8)
  207 FORMAT('Night ',A6,' SAAO 0.5-m uvby ',A8,1X)
C
C     Formats of error messages
C
  301 FORMAT(' Error: Star ',3A4,' is missing in table of objects')
  302 FORMAT(' Error in data sequence at time ',3I2)
C
      PRINT *,'SAAO322:
     *       Creating HEC22 type B data  Rel. 2: October 11, 2005'
C
      PRINT *,'Input file of SAAO Lucy UBV data?'
      READ(*,'(A)') INP
      PRINT *,'Output HEC22 data file?'
      READ(*,'(A)') OUT
      OPEN(1,FILE=INP,STATUS='OLD')
      OPEN(2,FILE=OUT,STATUS='NEW')
      OPEN(3,FILE='tabsaao.con',STATUS='OLD')
      OPEN(4,FILE='saao.cal',STATUS='OLD')
C
C
      READ(4,*) F
      READ(1,206) TEX
      WRITE(*,206) TEX
    1 READ(1,101) ZN
      IF(ZN.EQ.ZK(1)) GO TO 2
      BACKSPACE 1
      READ(1,206) TEX
      WRITE(*,206) TEX
      GO TO 1
C
    2 PRINT *,'Night No.? (max. 6 characters, e.g.: 25/92)'
      READ(*,'(A)') NGT
      PRINT *,'Observer? (max. 8 characters)'
      READ(*,'(A)') OBS
      WRITE(2,207) NGT,OBS
      WRITE(*,207) NGT,OBS
      PRINT *,'Comments on quality, weather etc. [max. 40 characters!]'
      PRINT *,'<                                     >'
      READ(*,206) TX
      WRITE(2,202) TX
      PRINT *,'Date of the beginning of night?'
      READ(*,*) D,M,J
      WRITE(2,203) D,M,J
C
C     Input of table of star names and specifications
C
      NN=0
      NOBS=0
    3 READ(3,102,END=7) HV,K,IF                                         Rel.2
      NN=NN+1
      DO 6 I=1,3
    6 S(NN,I)=HV(I)
      NS(NN)=K
      FIL(K)=IF                                                         Rel.2
      GO TO 3
C
C     Input of observations begins
C
    7 BACKSPACE 1
      READ(1,103,END=10) ZN,HV
      DO 4 J=1,NN
      DO 5 I=1,3
      IF(S(J,I).NE.HV(I)) GO TO 4
    5 CONTINUE
      N=NS(J)
      GO TO 8
    4 CONTINUE
      WRITE(2,301) HV
      STOP 13
C
C     Input of stellar or sky deflections
C
    8 READ(1,101) ZN
      IF(ZN.NE.ZK(3)) GO TO 8
   12 BACKSPACE 1
      READ(1,104) ZN,JH,JM,JS,IT,J,K,M,Q,L
      I=BAR(K-10)
      NO=N
      NOBS=NOBS+1
      IF(M.EQ.0) NO=-1
      TS=JS+5D-1*IT
      Q=Q/IT
      CALL DEAD(Q)
      IF(L.EQ.0) GO TO 11
      J=3*(FIL(NO)-1)+I                                                 Rel.2
      Q=Q*F(J)
   11 WRITE(2,204) I,NO,JH,JM,TS,Q
      READ(1,101) ZN
      IF(ZN.EQ.ZK(3)) GO TO 12
    9 READ(1,101,END=10) ZN
      IF(ZN.NE.ZK(1)) GO TO 9
      GO TO 7
C
   10 WRITE(2,205)
      STOP 7
      END
C
C
      SUBROUTINE DEAD(A)
C
C     CALCULATION OF THE DEAD-TIME CORRECTION
C
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON /DTC/ T
C
      IF(A.EQ.0D0) RETURN
      B=A
    1 C=B
      B=A*DEXP(T*B)
      IF(DABS((B-C)/B).GT.1.D-8) GO TO 1
      A=B
      RETURN
      END
